@@ -26,6 +26,7 @@ http://www.python.org/2.2.3/license.html"""
 #*****************************************************************************
 
 import bdb
+import functools
 import linecache
 import sys
 
@@ -38,8 +39,8 @@ from IPython.core.excolors import exception_colors
 has_pydb = False
 prompt = 'ipdb> '
 #We have to check this directly from sys.argv, config struct not yet available
-if '-pydb' in sys.argv:
-    try:        
+if '--pydb' in sys.argv:
+    try:
         import pydb
         if hasattr(pydb.pydb, "runl") and pydb.version>'1.17':
             # Version 1.17 is broken, and that's what ships with Ubuntu Edgy, so we
@@ -58,13 +59,21 @@ else:
 # Allow the set_trace code to operate outside of an ipython instance, even if
 # it does so with some limitations.  The rest of this support is implemented in
 # the Tracer constructor.
-def BdbQuit_excepthook(et,ev,tb):
+def BdbQuit_excepthook(et, ev, tb, excepthook=None):
+    """Exception hook which handles `BdbQuit` exceptions.
+
+    All other exceptions are processed using the `excepthook`
+    parameter.
+    """
     if et==bdb.BdbQuit:
         print 'Exiting Debugger.'
+    elif excepthook is not None:
+        excepthook(et, ev, tb)
     else:
+        # Backwards compatibility. Raise deprecation warning?
         BdbQuit_excepthook.excepthook_ori(et,ev,tb)
 
-def BdbQuit_IPython_excepthook(self,et,ev,tb):
+def BdbQuit_IPython_excepthook(self,et,ev,tb,tb_offset=None):
     print 'Exiting Debugger.'
 
 
@@ -104,11 +113,11 @@ class Tracer(object):
         """
 
         try:
-            ip = ipapi.get()
-        except:
+            ip = get_ipython()
+        except NameError:
             # Outside of ipython, we set our own exception hook manually
-            BdbQuit_excepthook.excepthook_ori = sys.excepthook
-            sys.excepthook = BdbQuit_excepthook
+            sys.excepthook = functools.partial(BdbQuit_excepthook,
+                                               excepthook=sys.excepthook)
             def_colors = 'NoColor'
             try:
                 # Limited tab completion support
@@ -123,6 +132,22 @@ class Tracer(object):
 
         if colors is None:
             colors = def_colors
+
+        # The stdlib debugger internally uses a modified repr from the `repr`
+        # module, that limits the length of printed strings to a hardcoded
+        # limit of 30 characters.  That much trimming is too aggressive, let's
+        # at least raise that limit to 80 chars, which should be enough for
+        # most interactive uses.
+        try:
+            from repr import aRepr
+            aRepr.maxstring = 80
+        except:
+            # This is only a user-facing convenience, so any error we encounter
+            # here can be warned about but can be otherwise ignored.  These
+            # printouts will tell us about problems if this API changes
+            import traceback
+            traceback.print_exc()
+
         self.debugger = Pdb(colors)
 
     def __call__(self):
@@ -130,7 +155,7 @@ class Tracer(object):
 
         This is similar to the pdb.set_trace() function from the std lib, but
         using IPython's enhanced debugger."""
-        
+
         self.debugger.set_trace(sys._getframe().f_back)
 
 
@@ -173,9 +198,9 @@ class Pdb(OldPdb):
             OldPdb.__init__(self,stdin=stdin,stdout=io.stdout)
         else:
             OldPdb.__init__(self,completekey,stdin,stdout)
-            
+
         self.prompt = prompt # The default prompt is '(Pdb)'
-        
+
         # IPython changes...
         self.is_pydb = has_pydb
 
@@ -207,7 +232,7 @@ class Pdb(OldPdb):
         # module and add a few attributes needed for debugging
         self.color_scheme_table = exception_colors()
 
-        # shorthands 
+        # shorthands
         C = coloransi.TermColors
         cst = self.color_scheme_table
 
@@ -250,11 +275,11 @@ class Pdb(OldPdb):
         self.shell.set_completer_frame(self.curframe)
 
     def new_do_quit(self, arg):
-        
+
         if hasattr(self, 'old_all_completions'):
             self.shell.Completer.all_completions=self.old_all_completions
-        
-        
+
+
         return OldPdb.do_quit(self, arg)
 
     do_q = do_quit = decorate_fn_with_doc(new_do_quit, OldPdb.do_quit)
@@ -288,9 +313,9 @@ class Pdb(OldPdb):
 
     def format_stack_entry(self, frame_lineno, lprefix=': ', context = 3):
         import linecache, repr
-        
+
         ret = []
-        
+
         Colors = self.color_scheme_table.active_colors
         ColorsNormal = Colors.Normal
         tpl_link = '%s%%s%s' % (Colors.filenameEm, ColorsNormal)
@@ -298,9 +323,9 @@ class Pdb(OldPdb):
         tpl_line = '%%s%s%%s %s%%s' % (Colors.lineno, ColorsNormal)
         tpl_line_em = '%%s%s%%s %s%%s%s' % (Colors.linenoEm, Colors.line,
                                             ColorsNormal)
-        
+
         frame, lineno = frame_lineno
-        
+
         return_value = ''
         if '__return__' in frame.f_locals:
             rv = frame.f_locals['__return__']
@@ -311,14 +336,14 @@ class Pdb(OldPdb):
         #s = filename + '(' + `lineno` + ')'
         filename = self.canonic(frame.f_code.co_filename)
         link = tpl_link % filename
-        
+
         if frame.f_code.co_name:
             func = frame.f_code.co_name
         else:
             func = "<lambda>"
-            
+
         call = ''
-        if func != '?':         
+        if func != '?':
             if '__args__' in frame.f_locals:
                 args = repr.repr(frame.f_locals['__args__'])
             else:
@@ -332,13 +357,13 @@ class Pdb(OldPdb):
         else:
             ret.append('  ')
         ret.append('%s(%s)%s\n' % (link,lineno,call))
-            
+
         start = lineno - 1 - context//2
         lines = linecache.getlines(filename)
-        start = max(start, 0)
         start = min(start, len(lines) - context)
+        start = max(start, 0)
         lines = lines[start : start + context]
-            
+
         for i,line in enumerate(lines):
             show_arrow = (start + 1 + i == lineno)
             linetpl = (frame is self.curframe or show_arrow) \
@@ -362,7 +387,7 @@ class Pdb(OldPdb):
         if lineno in self.get_file_breaks(filename):
             bps = self.get_breaks(filename, lineno)
             bp = bps[-1]
-        
+
         if bp:
             Colors = self.color_scheme_table.active_colors
             bp_mark = str(bp.number)
@@ -387,7 +412,7 @@ class Pdb(OldPdb):
         else:
             num = '%*s' % (numbers_width - len(bp_mark), str(lineno))
             line = tpl_line % (bp_mark_color + bp_mark, num, line)
-            
+
         return line
 
     def list_command_pydb(self, arg):
@@ -395,7 +420,7 @@ class Pdb(OldPdb):
         filename, first, last = OldPdb.parse_list_cmd(self, arg)
         if filename is not None:
             self.print_list_lines(filename, first, last)
-        
+
     def print_list_lines(self, filename, first, last):
         """The printing (as opposed to the parsing part of a 'list'
         command."""
@@ -461,19 +486,20 @@ class Pdb(OldPdb):
         """The debugger interface to magic_pdef"""
         namespaces = [('Locals', self.curframe.f_locals),
                       ('Globals', self.curframe.f_globals)]
-        self.shell.magic_pdef(arg, namespaces=namespaces)
+        self.shell.find_line_magic('pdef')(arg, namespaces=namespaces)
 
     def do_pdoc(self, arg):
         """The debugger interface to magic_pdoc"""
         namespaces = [('Locals', self.curframe.f_locals),
                       ('Globals', self.curframe.f_globals)]
-        self.shell.magic_pdoc(arg, namespaces=namespaces)
+        self.shell.find_line_magic('pdoc')(arg, namespaces=namespaces)
 
     def do_pinfo(self, arg):
         """The debugger equivalant of ?obj"""
         namespaces = [('Locals', self.curframe.f_locals),
                       ('Globals', self.curframe.f_globals)]
-        self.shell.magic_pinfo("pinfo %s" % arg, namespaces=namespaces)
+        self.shell.find_line_magic('pinfo')("pinfo %s" % arg,
+                                            namespaces=namespaces)
 
     def checkline(self, filename, lineno):
         """Check whether specified line seems to be executable.
@@ -496,7 +522,7 @@ class Pdb(OldPdb):
         #
         # End hack.  The rest of this method is copied verbatim from 2.6 pdb.py
         #######################################################################
-        
+
         if not line:
             print >>self.stdout, 'End of file'
             return 0
