@@ -4,16 +4,18 @@ from __future__ import print_function
 from collections import namedtuple
 import sys
 import time
+import uuid
 
 # System library imports
 from pygments.lexers import PythonLexer
+from IPython.external import qt
 from IPython.external.qt import QtCore, QtGui
 
 # Local imports
 from IPython.core.inputsplitter import InputSplitter, transform_classic_prompt
 from IPython.core.oinspect import call_tip
 from IPython.frontend.qt.base_frontend_mixin import BaseFrontendMixin
-from IPython.utils.traitlets import Bool, Instance
+from IPython.utils.traitlets import Bool, Instance, Unicode
 from bracket_matcher import BracketMatcher
 from call_tip_widget import CallTipWidget
 from completion_lexer import CompletionLexer
@@ -75,6 +77,9 @@ class FrontendWidget(HistoryConsoleWidget, BaseFrontendMixin):
     """ A Qt frontend for a generic Python kernel.
     """
 
+    # The text to show when the kernel is (re)started.
+    banner = Unicode()
+
     # An option and corresponding signal for overriding the default kernel
     # interrupt behavior.
     custom_interrupt = Bool(False)
@@ -86,6 +91,16 @@ class FrontendWidget(HistoryConsoleWidget, BaseFrontendMixin):
     custom_restart_kernel_died = QtCore.Signal(float)
     custom_restart_requested = QtCore.Signal()
 
+    # Whether to automatically show calltips on open-parentheses.
+    enable_calltips = Bool(True, config=True,
+        help="Whether to draw information calltips on open-parentheses.")
+
+    clear_on_kernel_restart = Bool(True, config=True,
+        help="Whether to clear the console when the kernel is restarted")
+
+    confirm_restart = Bool(True, config=True,
+        help="Whether to ask for user confirmation when restarting kernel")
+
     # Emitted when a user visible 'execute_request' has been submitted to the
     # kernel from the FrontendWidget. Contains the code to be executed.
     executing = QtCore.Signal(object)
@@ -95,9 +110,10 @@ class FrontendWidget(HistoryConsoleWidget, BaseFrontendMixin):
     executed = QtCore.Signal(object)
 
     # Emitted when an exit request has been received from the kernel.
-    exit_requested = QtCore.Signal()
-    
+    exit_requested = QtCore.Signal(object)
+
     # Protected class variables.
+    _transform_prompt = staticmethod(transform_classic_prompt)
     _CallTipRequest = namedtuple('_CallTipRequest', ['id', 'pos'])
     _CompletionRequest = namedtuple('_CompletionRequest', ['id', 'pos'])
     _ExecutionRequest = namedtuple('_ExecutionRequest', ['id', 'kind'])
@@ -108,9 +124,16 @@ class FrontendWidget(HistoryConsoleWidget, BaseFrontendMixin):
     #---------------------------------------------------------------------------
     # 'object' interface
     #---------------------------------------------------------------------------
-    
+
     def __init__(self, *args, **kw):
         super(FrontendWidget, self).__init__(*args, **kw)
+        # FIXME: remove this when PySide min version is updated past 1.0.7
+        # forcefully disable calltips if PySide is < 1.0.7, because they crash
+        if qt.QT_API == qt.QT_API_PYSIDE:
+            import PySide
+            if PySide.__version_info__ < (1,0,7):
+                self.log.warn("PySide %s < 1.0.7 detected, disabling calltips" % PySide.__version__)
+                self.enable_calltips = False
 
         # FrontendWidget protected variables.
         self._bracket_matcher = BracketMatcher(self._control)
@@ -122,6 +145,8 @@ class FrontendWidget(HistoryConsoleWidget, BaseFrontendMixin):
         self._input_splitter = self._input_splitter_class(input_mode='cell')
         self._kernel_manager = None
         self._request_info = {}
+        self._request_info['execute'] = {};
+        self._callback_dict = {}
 
         # Configure the ConsoleWidget.
         self.tab_width = 4
@@ -144,9 +169,9 @@ class FrontendWidget(HistoryConsoleWidget, BaseFrontendMixin):
         # Connect signal handlers.
         document = self._control.document()
         document.contentsChange.connect(self._document_contents_change)
-        
+
         # Set flag for whether we are connected via localhost.
-        self._local_kernel = kw.get('local_kernel', 
+        self._local_kernel = kw.get('local_kernel',
                                     FrontendWidget._local_kernel)
 
     #---------------------------------------------------------------------------
@@ -156,11 +181,16 @@ class FrontendWidget(HistoryConsoleWidget, BaseFrontendMixin):
     def copy(self):
         """ Copy the currently selected text to the clipboard, removing prompts.
         """
-        text = self._control.textCursor().selection().toPlainText()
-        if text:
-            lines = map(transform_classic_prompt, text.splitlines())
-            text = '\n'.join(lines)
-            QtGui.QApplication.clipboard().setText(text)
+        if self._page_control is not None and self._page_control.hasFocus():
+            self._page_control.copy()
+        elif self._control.hasFocus():
+            text = self._control.textCursor().selection().toPlainText()
+            if text:
+                lines = map(self._transform_prompt, text.splitlines())
+                text = '\n'.join(lines)
+                QtGui.QApplication.clipboard().setText(text)
+        else:
+            self.log.debug("frontend widget : unknown copy target")
 
     #---------------------------------------------------------------------------
     # 'ConsoleWidget' abstract interface
@@ -182,11 +212,11 @@ class FrontendWidget(HistoryConsoleWidget, BaseFrontendMixin):
         See parent class :meth:`execute` docstring for full details.
         """
         msg_id = self.kernel_manager.shell_channel.execute(source, hidden)
-        self._request_info['execute'] = self._ExecutionRequest(msg_id, 'user')
+        self._request_info['execute'][msg_id] = self._ExecutionRequest(msg_id, 'user')
         self._hidden = hidden
         if not hidden:
             self.executing.emit(source)
-        
+
     def _prompt_started_hook(self):
         """ Called immediately after a new prompt is displayed.
         """
@@ -234,6 +264,14 @@ class FrontendWidget(HistoryConsoleWidget, BaseFrontendMixin):
                 break
         return menu
 
+    def request_interrupt_kernel(self):
+        if self._executing:
+            self.interrupt_kernel()
+
+    def request_restart_kernel(self):
+        message = 'Are you sure you want to restart the kernel?'
+        self.restart_kernel(message, now=False)
+
     def _event_filter_console_keypress(self, event):
         """ Reimplemented for execution interruption and smart backspace.
         """
@@ -241,12 +279,11 @@ class FrontendWidget(HistoryConsoleWidget, BaseFrontendMixin):
         if self._control_key_down(event.modifiers(), include_command=False):
 
             if key == QtCore.Qt.Key_C and self._executing:
-                self.interrupt_kernel()
+                self.request_interrupt_kernel()
                 return True
 
             elif key == QtCore.Qt.Key_Period:
-                message = 'Are you sure you want to restart the kernel?'
-                self.restart_kernel(message, now=False)
+                self.request_restart_kernel()
                 return True
 
         elif not event.modifiers() & QtCore.Qt.AltModifier:
@@ -280,6 +317,7 @@ class FrontendWidget(HistoryConsoleWidget, BaseFrontendMixin):
     def _handle_complete_reply(self, rep):
         """ Handle replies for tab completion.
         """
+        self.log.debug("complete: %s", rep.get('content', ''))
         cursor = self._get_cursor()
         info = self._request_info.get('complete')
         if info and info.id == rep['parent_header']['msg_id'] and \
@@ -288,12 +326,74 @@ class FrontendWidget(HistoryConsoleWidget, BaseFrontendMixin):
             cursor.movePosition(QtGui.QTextCursor.Left, n=len(text))
             self._complete_with_items(cursor, rep['content']['matches'])
 
+    def _silent_exec_callback(self, expr, callback):
+        """Silently execute `expr` in the kernel and call `callback` with reply
+
+        the `expr` is evaluated silently in the kernel (without) output in
+        the frontend. Call `callback` with the
+        `repr <http://docs.python.org/library/functions.html#repr> `_ as first argument
+
+        Parameters
+        ----------
+        expr : string
+            valid string to be executed by the kernel.
+        callback : function
+            function accepting one argument, as a string. The string will be
+            the `repr` of the result of evaluating `expr`
+
+        The `callback` is called with the `repr()` of the result of `expr` as
+        first argument. To get the object, do `eval()` on the passed value.
+
+        See Also
+        --------
+        _handle_exec_callback : private method, deal with calling callback with reply
+
+        """
+
+        # generate uuid, which would be used as an indication of whether or
+        # not the unique request originated from here (can use msg id ?)
+        local_uuid = str(uuid.uuid1())
+        msg_id = self.kernel_manager.shell_channel.execute('',
+            silent=True, user_expressions={ local_uuid:expr })
+        self._callback_dict[local_uuid] = callback
+        self._request_info['execute'][msg_id] = self._ExecutionRequest(msg_id, 'silent_exec_callback')
+
+    def _handle_exec_callback(self, msg):
+        """Execute `callback` corresponding to `msg` reply, after ``_silent_exec_callback``
+
+        Parameters
+        ----------
+        msg : raw message send by the kernel containing an `user_expressions`
+                and having a 'silent_exec_callback' kind.
+
+        Notes
+        -----
+        This function will look for a `callback` associated with the
+        corresponding message id. Association has been made by
+        `_silent_exec_callback`. `callback` is then called with the `repr()`
+        of the value of corresponding `user_expressions` as argument.
+        `callback` is then removed from the known list so that any message
+        coming again with the same id won't trigger it.
+
+        """
+
+        user_exp = msg['content'].get('user_expressions')
+        if not user_exp:
+            return
+        for expression in user_exp:
+            if expression in self._callback_dict:
+                self._callback_dict.pop(expression)(user_exp[expression])
+
     def _handle_execute_reply(self, msg):
         """ Handles replies for code execution.
         """
-        info = self._request_info.get('execute')
-        if info and info.id == msg['parent_header']['msg_id'] and \
-                info.kind == 'user' and not self._hidden:
+        self.log.debug("execute: %s", msg.get('content', ''))
+        msg_id = msg['parent_header']['msg_id']
+        info = self._request_info['execute'].get(msg_id)
+        # unset reading flag, because if execute finished, raw_input can't
+        # still be pending.
+        self._reading = False
+        if info and info.kind == 'user' and not self._hidden:
             # Make sure that all output from the SUB channel has been processed
             # before writing a new prompt.
             self.kernel_manager.sub_channel.flush()
@@ -310,15 +410,22 @@ class FrontendWidget(HistoryConsoleWidget, BaseFrontendMixin):
                 self._process_execute_ok(msg)
             elif status == 'error':
                 self._process_execute_error(msg)
-            elif status == 'abort':
+            elif status == 'aborted':
                 self._process_execute_abort(msg)
 
             self._show_interpreter_prompt_for_reply(msg)
             self.executed.emit(msg)
+            self._request_info['execute'].pop(msg_id)
+        elif info and info.kind == 'silent_exec_callback' and not self._hidden:
+            self._handle_exec_callback(msg)
+            self._request_info['execute'].pop(msg_id)
+        else:
+            super(FrontendWidget, self)._handle_execute_reply(msg)
 
     def _handle_input_request(self, msg):
         """ Handle requests for raw_input.
         """
+        self.log.debug("input: %s", msg.get('content', ''))
         if self._hidden:
             raise RuntimeError('Request for raw input during hidden execution.')
 
@@ -328,11 +435,15 @@ class FrontendWidget(HistoryConsoleWidget, BaseFrontendMixin):
 
         def callback(line):
             self.kernel_manager.stdin_channel.input(line)
+        if self._reading:
+            self.log.debug("Got second input request, assuming first was interrupted.")
+            self._reading = False
         self._readline(msg['content']['prompt'], callback=callback)
 
     def _handle_kernel_died(self, since_last_heartbeat):
         """ Handle the kernel's death by asking if the user wants to restart.
         """
+        self.log.debug("kernel died: %s", since_last_heartbeat)
         if self.custom_restart:
             self.custom_restart_kernel_died.emit(since_last_heartbeat)
         else:
@@ -345,6 +456,7 @@ class FrontendWidget(HistoryConsoleWidget, BaseFrontendMixin):
     def _handle_object_info_reply(self, rep):
         """ Handle replies for call tips.
         """
+        self.log.debug("oinfo: %s", rep.get('content', ''))
         cursor = self._get_cursor()
         info = self._request_info.get('call_tip')
         if info and info.id == rep['parent_header']['msg_id'] and \
@@ -368,6 +480,7 @@ class FrontendWidget(HistoryConsoleWidget, BaseFrontendMixin):
     def _handle_pyout(self, msg):
         """ Handle display hook output.
         """
+        self.log.debug("pyout: %s", msg.get('content', ''))
         if not self._hidden and self._is_from_this_session(msg):
             text = msg['content']['data']
             self._append_plain_text(text + '\n', before_prompt=True)
@@ -375,22 +488,24 @@ class FrontendWidget(HistoryConsoleWidget, BaseFrontendMixin):
     def _handle_stream(self, msg):
         """ Handle stdout, stderr, and stdin.
         """
+        self.log.debug("stream: %s", msg.get('content', ''))
         if not self._hidden and self._is_from_this_session(msg):
             # Most consoles treat tabs as being 8 space characters. Convert tabs
             # to spaces so that output looks as expected regardless of this
             # widget's tab width.
             text = msg['content']['data'].expandtabs(8)
-            
+
             self._append_plain_text(text, before_prompt=True)
             self._control.moveCursor(QtGui.QTextCursor.End)
 
     def _handle_shutdown_reply(self, msg):
         """ Handle shutdown signal, only if from other console.
         """
+        self.log.debug("shutdown: %s", msg.get('content', ''))
         if not self._hidden and not self._is_from_this_session(msg):
             if self._local_kernel:
                 if not msg['content']['restart']:
-                    sys.exit(0)
+                    self.exit_requested.emit(self)
                 else:
                     # we just got notified of a restart!
                     time.sleep(0.25) # wait 1/4 sec to reset
@@ -405,8 +520,10 @@ class FrontendWidget(HistoryConsoleWidget, BaseFrontendMixin):
                         "Close the Console?",
                         QtGui.QMessageBox.Yes,QtGui.QMessageBox.No)
                     if reply == QtGui.QMessageBox.Yes:
-                        sys.exit(0)
+                        self.exit_requested.emit(self)
                 else:
+                    # XXX: remove message box in favor of using the
+                    # clear_on_kernel_restart setting?
                     reply = QtGui.QMessageBox.question(self, title,
                         "Kernel has been reset. Clear the Console?",
                         QtGui.QMessageBox.Yes,QtGui.QMessageBox.No)
@@ -417,10 +534,10 @@ class FrontendWidget(HistoryConsoleWidget, BaseFrontendMixin):
                         self.reset()
 
     def _started_channels(self):
-        """ Called when the KernelManager channels have started listening or 
+        """ Called when the KernelManager channels have started listening or
             when the frontend is assigned an already listening KernelManager.
         """
-        self.reset()
+        self.reset(clear=True)
 
     #---------------------------------------------------------------------------
     # 'FrontendWidget' public interface
@@ -440,27 +557,46 @@ class FrontendWidget(HistoryConsoleWidget, BaseFrontendMixin):
 
     def interrupt_kernel(self):
         """ Attempts to interrupt the running kernel.
+        
+        Also unsets _reading flag, to avoid runtime errors
+        if raw_input is called again.
         """
         if self.custom_interrupt:
+            self._reading = False
             self.custom_interrupt_requested.emit()
         elif self.kernel_manager.has_kernel:
+            self._reading = False
             self.kernel_manager.interrupt_kernel()
         else:
             self._append_plain_text('Kernel process is either remote or '
                                     'unspecified. Cannot interrupt.\n')
 
-    def reset(self):
-        """ Resets the widget to its initial state. Similar to ``clear``, but
-            also re-writes the banner and aborts execution if necessary.
-        """ 
+    def reset(self, clear=False):
+        """ Resets the widget to its initial state if ``clear`` parameter or
+        ``clear_on_kernel_restart`` configuration setting is True, otherwise
+        prints a visual indication of the fact that the kernel restarted, but
+        does not clear the traces from previous usage of the kernel before it
+        was restarted.  With ``clear=True``, it is similar to ``%clear``, but
+        also re-writes the banner and aborts execution if necessary.
+        """
         if self._executing:
             self._executing = False
-            self._request_info['execute'] = None
+            self._request_info['execute'] = {}
         self._reading = False
         self._highlighter.highlighting_on = False
 
-        self._control.clear()
-        self._append_plain_text(self._get_banner())
+        if self.clear_on_kernel_restart or clear:
+            self._control.clear()
+            self._append_plain_text(self.banner)
+        else:
+            self._append_plain_text("# restarting kernel...")
+            self._append_html("<hr><br>")
+            # XXX: Reprinting the full banner may be too much, but once #1680 is
+            # addressed, that will mitigate it.
+            #self._append_plain_text(self.banner)
+        # update output marker for stdout/stderr, so that startup
+        # messages appear after banner:
+        self._append_before_prompt_pos = self._get_cursor().position()
         self._show_interpreter_prompt()
 
     def restart_kernel(self, message, now=False):
@@ -482,15 +618,23 @@ class FrontendWidget(HistoryConsoleWidget, BaseFrontendMixin):
             # Prompt the user to restart the kernel. Un-pause the heartbeat if
             # they decline. (If they accept, the heartbeat will be un-paused
             # automatically when the kernel is restarted.)
-            buttons = QtGui.QMessageBox.Yes | QtGui.QMessageBox.No
-            result = QtGui.QMessageBox.question(self, 'Restart kernel?',
-                                                message, buttons)
-            if result == QtGui.QMessageBox.Yes:
+            if self.confirm_restart:
+                buttons = QtGui.QMessageBox.Yes | QtGui.QMessageBox.No
+                result = QtGui.QMessageBox.question(self, 'Restart kernel?',
+                                                    message, buttons)
+                do_restart = result == QtGui.QMessageBox.Yes
+            else:
+                # confirm_restart is False, so we don't need to ask user
+                # anything, just do the restart
+                do_restart = True
+            if do_restart:
                 try:
                     self.kernel_manager.restart_kernel(now=now)
                 except RuntimeError:
                     self._append_plain_text('Kernel started externally. '
-                                            'Cannot restart.\n')
+                                            'Cannot restart.\n',
+                                            before_prompt=True
+                                            )
                 else:
                     self.reset()
             else:
@@ -498,7 +642,9 @@ class FrontendWidget(HistoryConsoleWidget, BaseFrontendMixin):
 
         else:
             self._append_plain_text('Kernel process is either remote or '
-                                    'unspecified. Cannot restart.\n')
+                                    'unspecified. Cannot restart.\n',
+                                    before_prompt=True
+                                    )
 
     #---------------------------------------------------------------------------
     # 'FrontendWidget' protected interface
@@ -508,6 +654,8 @@ class FrontendWidget(HistoryConsoleWidget, BaseFrontendMixin):
         """ Shows a call tip, if appropriate, at the current cursor location.
         """
         # Decide if it makes sense to show a call tip
+        if not self.enable_calltips:
+            return False
         cursor = self._get_cursor()
         cursor.movePosition(QtGui.QTextCursor.Left)
         if cursor.document().characterAt(cursor.position()) != '(':
@@ -533,25 +681,18 @@ class FrontendWidget(HistoryConsoleWidget, BaseFrontendMixin):
                 '.'.join(context),                       # text
                 self._get_input_buffer_cursor_line(),    # line
                 self._get_input_buffer_cursor_column(),  # cursor_pos
-                self.input_buffer)                       # block 
+                self.input_buffer)                       # block
             pos = self._get_cursor().position()
             info = self._CompletionRequest(msg_id, pos)
             self._request_info['complete'] = info
 
-    def _get_banner(self):
-        """ Gets a banner to display at the beginning of a session.
-        """
-        banner = 'Python %s on %s\nType "help", "copyright", "credits" or ' \
-            '"license" for more information.'
-        return banner % (sys.version, sys.platform)
-
     def _get_context(self, cursor=None):
         """ Gets the context for the specified cursor (or the current cursor
             if none is specified).
         """
         if cursor is None:
             cursor = self._get_cursor()
-        cursor.movePosition(QtGui.QTextCursor.StartOfBlock, 
+        cursor.movePosition(QtGui.QTextCursor.StartOfBlock,
                             QtGui.QTextCursor.KeepAnchor)
         text = cursor.selection().toPlainText()
         return self._completion_lexer.get_context(text)
@@ -571,13 +712,13 @@ class FrontendWidget(HistoryConsoleWidget, BaseFrontendMixin):
         if content['ename']=='SystemExit':
             keepkernel = content['evalue']=='-k' or content['evalue']=='True'
             self._keep_kernel_on_exit = keepkernel
-            self.exit_requested.emit()
+            self.exit_requested.emit(self)
         else:
             traceback = ''.join(content['traceback'])
             self._append_plain_text(traceback)
 
     def _process_execute_ok(self, msg):
-        """ Process a reply for a successful execution equest.
+        """ Process a reply for a successful execution request.
         """
         payload = msg['content']['payload']
         for item in payload:
@@ -615,3 +756,12 @@ class FrontendWidget(HistoryConsoleWidget, BaseFrontendMixin):
         document = self._control.document()
         if position == self._get_cursor().position():
             self._call_tip()
+
+    #------ Trait default initializers -----------------------------------------
+
+    def _banner_default(self):
+        """ Returns the standard Python banner.
+        """
+        banner = 'Python %s on %s\nType "help", "copyright", "credits" or ' \
+            '"license" for more information.'
+        return banner % (sys.version, sys.platform)
