@@ -155,8 +155,7 @@ class _PrettyPrinterBase(object):
         """like begin_group / end_group but for the with statement."""
         self.begin_group(indent, open)
         try:
-            with self.indent(indent):
-                yield
+            yield
         finally:
             self.end_group(indent, close)
 
@@ -337,17 +336,27 @@ class RepresentationPrinter(PrettyPrinter):
                 pass
             else:
                 return printer(obj, self, cycle)
-            # Next look for type_printers.
+            # Next walk the mro and check for either:
+            #   1) a registered printer
+            #   2) a _repr_pretty_ method
             for cls in _get_mro(obj_class):
                 if cls in self.type_pprinters:
+                    # printer registered in self.type_pprinters
                     return self.type_pprinters[cls](obj, self, cycle)
                 else:
+                    # deferred printer
                     printer = self._in_deferred_types(cls)
                     if printer is not None:
                         return printer(obj, self, cycle)
-            # Finally look for special method names.
-            if hasattr(obj_class, '_repr_pretty_'):
-                return obj_class._repr_pretty_(obj, self, cycle)
+                    else:
+                        # Finally look for special method names.
+                        # Some objects automatically create any requested
+                        # attribute. Try to ignore most of them by checking for
+                        # callability.
+                        if '_repr_pretty_' in obj_class.__dict__:
+                            meth = obj_class._repr_pretty_
+                            if callable(meth):
+                                return meth(obj, self, cycle)
             return _default_pprint(obj, self, cycle)
         finally:
             self.end_group()
@@ -454,8 +463,10 @@ class GroupQueue(object):
         except ValueError:
             pass
 
-
-_baseclass_reprs = (object.__repr__, types.InstanceType.__repr__)
+try:
+    _baseclass_reprs = (object.__repr__, types.InstanceType.__repr__)
+except AttributeError:  # Python 3
+    _baseclass_reprs = (object.__repr__,)
 
 
 def _default_pprint(obj, p, cycle):
@@ -591,10 +602,16 @@ def _re_pattern_pprint(obj, p, cycle):
 
 def _type_pprint(obj, p, cycle):
     """The pprint for classes and types."""
-    if obj.__module__ in ('__builtin__', 'exceptions'):
+    try:
+        mod = obj.__module__
+    except AttributeError:
+        # Heap allocated types might not have the module attribute.
+        return p.text(obj.__name__)
+
+    if mod in ('__builtin__', 'exceptions'):
         name = obj.__name__
     else:
-        name = obj.__module__ + '.' + obj.__name__
+        name = mod + '.' + obj.__name__
     p.text(name)
 
 
@@ -614,7 +631,7 @@ def _function_pprint(obj, p, cycle):
 
 def _exception_pprint(obj, p, cycle):
     """Base pprint for all exceptions."""
-    if obj.__class__.__module__ == 'exceptions':
+    if obj.__class__.__module__ in ('exceptions', 'builtins'):
         name = obj.__class__.__name__
     else:
         name = '%s.%s' % (
@@ -622,7 +639,7 @@ def _exception_pprint(obj, p, cycle):
             obj.__class__.__name__
         )
     step = len(name) + 1
-    p.begin_group(step, '(')
+    p.begin_group(step, name + '(')
     for idx, arg in enumerate(getattr(obj, 'args', ())):
         if idx:
             p.text(',')
@@ -648,23 +665,33 @@ _type_pprinters = {
     tuple:                      _seq_pprinter_factory('(', ')', tuple),
     list:                       _seq_pprinter_factory('[', ']', list),
     dict:                       _dict_pprinter_factory('{', '}', dict),
-    types.DictProxyType:        _dict_pprinter_factory('<dictproxy {', '}>'),
+    
     set:                        _seq_pprinter_factory('set([', '])', set),
     frozenset:                  _seq_pprinter_factory('frozenset([', '])', frozenset),
     super:                      _super_pprint,
     _re_pattern_type:           _re_pattern_pprint,
     type:                       _type_pprint,
-    types.ClassType:            _type_pprint,
     types.FunctionType:         _function_pprint,
     types.BuiltinFunctionType:  _function_pprint,
     types.SliceType:            _repr_pprint,
     types.MethodType:           _repr_pprint,
-    xrange:                     _repr_pprint,
+    
     datetime.datetime:          _repr_pprint,
     datetime.timedelta:         _repr_pprint,
     _exception_base:            _exception_pprint
 }
 
+try:
+    _type_pprinters[types.DictProxyType] = _dict_pprinter_factory('<dictproxy {', '}>')
+    _type_pprinters[types.ClassType] = _type_pprint
+except AttributeError: # Python 3
+    pass
+    
+try:
+    _type_pprinters[xrange] = _repr_pprint
+except NameError:
+    _type_pprinters[range] = _repr_pprint
+
 #: printers for types specified by name
 _deferred_type_pprinters = {
 }
